/* NIC identification / PROTOCOL.INI file builder for eCS */
/* Many thanks go to Chuck McKinnis (http://pws.prserv.net/mckinnis/nicpak/) for creating the NIC PAK for OS/2, his code is used in this script */
/* This script was put together by Paltiel Goldstein ( paltiel@cybercomm.net ) */
/* Use at your own risk */

/* Version 0.23.3 */
version = '0.23.3'

/* SIGNAL ON FAILURE
SIGNAL ON ERROR */

Call Rxfuncadd 'SysLoadFuncs','Rexxutil','SysLoadFuncs'
Call SysLoadFuncs

call sysCls

SAY '[STARTING] Preparing to run NIC identification / PROTOCOL.INI file builder'

FLAG_force_drive_choice = 0

/* PLEASE NOTE!  The ramdrive variable must -not- have a trailing backslash */
ramdrive = value("RAM_DRIVELETTER",,"OS2ENVIRONMENT")
if (ramdrive<>"") then ramdrive = substr(ramdrive,1,1) || ":"
IF (ramdrive = '') THEN DO
   SAY '[WARNING]  Unable to identify ramdrive'
   SAY '           ( via RAM_DRIVELETTER environment variable )'
   SAY '           User input required to identify writable temporary location with'
   SAY '           nicdrv.lst and driver files ( *.NIF & *.OS2 )'
   FLAG_force_drive_choice = 1
END

NIC_Driver_name =       'nicdrv.os2'
NIF_File_name =         'nicdrv.nif'
protocolINI_name =      'protocol.ini'
dest_NIC_Driver_name =  ramdrive || '\' || NIC_Driver_name
dest_NIF_File_name =    ramdrive || '\' || NIF_File_name
dest_protocolINI_name = ramdrive || '\' || protocolINI_name

/***** Do not edit variables below this line *****/
source_NIC_Driver =     ''
source_NIF_File =       ''
drivername = ''
driverSectionName = ''

/* Locate the list of NIC IDs */
Parse Source . . this_cmd .
current_dir = Left(this_cmd, Lastpos('\', this_cmd) - 1)
Call Directory(current_dir)
nic_lst = current_dir || '\nicdrv.lst'
IF STREAM(nic_lst,'C','QUERY EXISTS')='' THEN DO
  SAY '[FATAL ERROR] Unable to find list of NIC driver IDs'
  SAY 'Exiting..'
  exit
END

logDir = ''
logFile = ''

call setupLogs

call writeLog '[NOTICE]   Revision ' || version

/* Find \IBMCOM\MACS */
ibm_macs = SysSearchPath('path', 'mpts.exe')
If ibm_macs <> '' Then
   Do
      ibm_macs = Left(ibm_macs, Lastpos('\', ibm_macs) -1)
      ibm_macs = ibm_macs || '\MACS'
      call writeLog '[NOTICE]   Assuming that driver files are located in "' || ibm_macs || '"'
   End
ELSE DO
   path = translate(value('path', ,'os2environment'))
   ipos = pos("\IBMCOM",path)
   ibm_macs=""
   if (ipos>2) then do
      ibm_macs = substr(path,ipos-2,2) || "\ibmcom\macs"
      rc = SysFileTree(ibm_macs, 'files.', 'd')
      if (files.0=0) then ibm_macs = ""
      drop files.
   END
   IF ibm_macs <> '' THEN DO
      call writeLog '[NOTICE]   Assuming that driver files are located in "' || ibm_macs || '"'
   END
   ELSE DO
      call writeLog '[FATAL ERROR] Can not identify location of \MACS directory for network driver files'
      call writeLog 'Exiting..'
      Exit
   END
END

/* Find \IBMCOM\PROTOCOL */
ibm_protocol = SysSearchPath('path', 'mpts.exe')
If ibm_protocol <> '' Then
   Do
      ibm_protocol = Left(ibm_protocol, Lastpos('\', ibm_macs) -1)
      ibm_protocol = ibm_protocol || '\protocol'
      call writeLog '[NOTICE]   Assuming that protocol files are located in "' || ibm_protocol || '"'
   End
ELSE DO
   path = translate(value('path', ,'os2environment'))
   ipos = pos("\IBMCOM",path)
   ibm_protocol=""
   if (ipos>2) then do
      ibm_protocol = substr(path,ipos-2,2) || "\ibmcom\protocol"
      rc = SysFileTree(ibm_protocol, 'files.', 'd')
      if (files.0=0) then ibm_protocol = ""
      drop files.
   END
   IF ibm_protocol <> '' THEN DO
      call writeLog '[NOTICE]   Assuming that protocol files are located in "' || ibm_protocol || '"'
   END
   ELSE DO
      call writeLog '[FATAL ERROR] Can not identify location of \PROTOCOL directory for network driver files'
      call writeLog 'Exiting..'
      Exit
   END
END

FLAG_key_pressed = 0

IF (FLAG_force_drive_choice = 0) THEN DO
   call writeLog '[OPTIONAL] Press any key to manually identify where drivers for your network'
   call writeLog '           adapter are located'
   returnValue = WaitTime(20)
   IF (returnValue <> 0) THEN DO
      FLAG_force_drive_choice = 1
      FLAG_key_pressed = 1
   END
END

IF (FLAG_force_drive_choice <> 0) THEN DO
   IF FLAG_key_pressed = 1 THEN call writeLog '(key pressed)'
   call writeLog '[NOTICE]   Please wait, identifying available drive letters in your system..'
   /* Assume that the user wants to choose an optional place for the nicdrv.lst & NIC drivers */
   /* Find out what drive letters are in the system, then see if they're ready or not */
   available_drives = sysDriveMap('A:', 'LOCAL')
   network_drives = sysDriveMap('A:', 'REMOTE')
   IF (available_drives <> '') THEN call writeLog 'available_drives: ' || available_drives
   IF (network_drives <> '') THEN call writeLog 'network_drives:   ' || network_drives
   available_drives = TRANSLATE(available_drives, '', ':')
   network_drives = TRANSLATE(network_drives, '', ':')
   alldrives = available_drives || network_drives
   call writeLog '[INPUT]    Please press the key corresponding to the drive letter that contains'
   call writeLog '           a "nicdrv.lst" file and drivers for your network adapter'
   call writeLog '[NOTE]     Files must be located in the root of that drive'
   call writeLog
   local_nic_lst = current_dir || '\nicdrv.lst'
   IF (ibm_macs <>'') THEN DO
      call writeLog '[NOTE]     Do you want to use driver files from "' || ibm_macs || '"'
      call writeLog '           and the "nicdrv.lst" file from "' || current_dir || '"?'
      call writeLog '           press ''' || '1' || ''''
   END
   FLAG_do_loop = 1
   DO WHILE (FLAG_do_loop = 1)
      IF (chars() <> 0) THEN DO
         temp = sysgetkey('NOECHO')
         temp = translate(temp)
         FLAG_valid_key_pressed = 0
         /* Check if they pressed a valid key */
         pos = wordpos(temp, alldrives)
         IF (pos <> 0) THEN DO
            FLAG_valid_key_pressed = 1
         END
         /* Special case if they press the '1' key */
         IF (temp = '1') THEN DO
            /* Drive chosen is where this script is located */
            call writeLog '[NOTICE]   You chose to use driver files from "' || ibm_macs || '"'
            call writeLog '           and the "nicdrv.lst" file from "' || current_dir || '"'
            new_nic_lst = current_dir || '\nicdrv.lst'
            IF STREAM(new_nic_lst,'C','QUERY EXISTS')<>'' THEN DO
               call writeLog '[NOTICE]   Using "' || current_dir || '\nicdrv.lst" file'
               ramdrive = current_dir
               call writeLog '[NOTICE]   Placing all output files in "' || ramdrive || '"'
               dest_NIC_Driver_name =  ramdrive || '\' || NIC_Driver_name
               dest_NIF_File_name =    ramdrive || '\' || NIF_File_name
               dest_protocolINI_name = ramdrive || '\' || protocolINI_name
               FLAG_do_loop = 0
            END
        END
         IF (FLAG_valid_key_pressed = 1) THEN DO
            call writeLog '[NOTICE]   Please wait, checking drive ' || temp || ':..'
            new_nic_lst = temp || ':\nicdrv.lst'
            IF STREAM(new_nic_lst,'C','QUERY EXISTS')<>'' THEN DO
               call writeLog '[NOTICE]   Using "' || temp || ':\nicdrv.lst" file'
               nic_lst = new_nic_lst
               ibm_macs = temp || ':'
               call writeLog '[NOTICE]   User indicated to use driver files located in "' || ibm_macs || '\"'
               FLAG_do_loop = 0
               IF (ramdrive = '') THEN DO
                  ramdrive = temp || ':'
                  call writeLog '[NOTICE]   Placing all output files in ' || ramdrive || '\'
                  dest_NIC_Driver_name =  ramdrive || '\' || NIC_Driver_name
                  dest_NIF_File_name =    ramdrive || '\' || NIF_File_name
                  dest_protocolINI_name = ramdrive || '\' || protocolINI_name
               END
            END
            ELSE DO
               call writeLog '[WARNING]  Unable to find "' || temp || ':\nicdrv.lst" file'
               /* SAY 'current_dir: ' || current_dir */
               /* SAY 'temp:        ' || temp */
               call writeLog '[INPUT]    Please press the key corresponding to the drive letter that contains'
               call writeLog '           a "nicdrv.lst" file and drivers for your network adapter'
            END
         END
         ELSE DO
            IF (temp <> '1') THEN DO
               call writeLog '[WARNING]  Drive ' || temp || ': is not a valid choice'
            END
         END
      END
      ELSE call sysSleep 1
   END
END

call writeLog '[NOTICE]   Please wait, identifying NIC adapters in your system..'
call writeLog
Call Build_device_list                        /* build device id list */

/* select a NIC */
If mfg_no.0 <> 0 Then
   Do
      call writeLog 'Detected following NIC(s):'
      nic_select = Select_nic('D')             /* select a NIC driver */
   End
Else
   Do
      call writeLog '[WARNING]  Unable to detect any known NIC adapters in this system'
      IF (bus_device.0 = 0) THEN DO
         call writeLog '[WARNING]  No PCI devices of type 200 detected in this system'
      END
      DO I = 1 To bus_device.0
         component_string = bus_device.I
         /* SAY 'bus_device.' || I || ': ' || component_string */
         mfg_id = left(component_string, 4)
         /* SAY 'mfg_id:       ' || mfg_id */
         dev_id = right(component_string,4)
         /* SAY 'dev_id:       ' || dev_id */
         /* blah */
         pci_lst = current_dir || '\pcidevs.txt'
         IF STREAM(pci_lst,'C','QUERY EXISTS')='' THEN DO
            call writeLog '[WARNING] Unable to find list of PCI driver IDs ( pcidevs.txt )'
            call writeLog 'Exiting..'
            exit
         END
         parse value SysCurPos() with x y
         IF (queued() <> 0) THEN DO
            call writeLog '[NOTICE]   Preparing to identify components via PCI ID..'
            call writeLog '[NOTE]     This may take up to ten minutes'
            DO N = 1 Until Queued() = 0
               /* Flush out the Rexx Queue */
               Parse Pull line
            END
         END
         ELSE DO
            call writeLog '[NOTICE]   Preparing to identify components via PCI ID..'
         END
         call writeLog '[NOTICE]   Preparation complete'
         call writeLog '[NOTICE]   Please wait, reading in list of PCI devices..'
         call writeLog '[NOTE]     This may take up to ten minutes'
         /*
         pos = 0
         parse value SysCurPos() with x y
         DO WHILE (lines(pci_lst) <> 0)
            pos = pos + 1
            temp = linein(pci_lst, pos)
            push temp
            call SysCurPos x, y
            SAY pos || ' lines read'
            parse value SysCurPos() with newX newY
            IF (newX = x) THEN DO
               x = x - 1
            END           
         END
         call SysCurPos x, y
         */
         /* '@rxqueue < ' || pci_lst */
         '@type ' || pci_lst || ' | rxqueue'
         call writeLog '[NOTICE]   Completed reading list (' || queued() || ' lines read)'
         FLAG_loop = 1
         FLAG_found_vendor = 0
         FLAG_found_device = 0
         parse value SysCurPos() with x y
         IF queued() = 0 THEN DO
            FLAG_loop = 0
         END
         ELSE DO
            call writeLog '[NOTICE]   Please wait, searching for vendor id "' || mfg_id || '"'
            parse value SysCurPos() with x y
         END
         call SysCurPos x, y
         DO WHILE (FLAG_loop = 1)
            IF (queued() <> 0) THEN Parse Pull current_line
            /* SAY 'current_line: ' || current_line */
            first_char = left(current_line, 1)
            /* SAY 'first_char: ' || first_char */
            IF (first_char = 'V') THEN DO
               current_line = right(current_line, length(current_line) - 2)
               vendor_id = left(current_line, 4)
               vendor_id = strip(vendor_id)
               vendor_id = left(vendor_id, 4)
               vendor_id = translate(vendor_id)
               call SysCurPos x, y
               SAY 'vendor_id: ' || vendor_id
               parse value SysCurPos() with newX newY
               IF (x = newX) THEN DO
                  x = x - 1
                  call SysCurPos x, y
               END
               IF (vendor_id = mfg_id) THEN DO
                  vendor_name = right(current_line, length(current_line) - 5)
                  vendor_name = strip(vendor_name)
                  /* SAY 'Found vendor ID ' || mfg_id || ' (' || vendor_name || ')' */
                  FLAG_found_vendor = 1
                  DO WHILE (FLAG_loop = 1)
                     IF (queued() <> 0) THEN Parse Pull current_line
                     call SysCurPos x, y
                     first_char = left(current_line, 1)
                     IF (first_char = 'D') THEN DO
                        current_line = right(current_line, length(current_line) - 2)
                        device_id = left(current_line, 4)
                        device_id = strip(device_id)
                        device_id = left(device_id, 4)
                        device_id = translate(device_id)
                        /* Look for a matching device ID */
                        IF (device_id = dev_id) THEN DO
                           /* Found it! */
                           device_name = right(current_line, length(current_line) - 5)
                           device_name = strip(device_name)
                           /* SAY 'Found device ID ' || dev_id || ' (' || device_name || ')' */
                           FLAG_found_device = 1
                           call Beep 500, 100
                        END
                     END
                     IF (first_char = 'V') THEN DO
                        /* We've searched through all the devices that the manufacturer makes */
                        /* terminate the search */
                        FLAG_loop = 0
                     END
                  END
               END
            END
            IF (queued() < 1) THEN DO
               FLAG_loop = 0
            END
      END
      IF (FLAG_found_vendor = 1) THEN DO
         IF (FLAG_found_device = 1) THEN DO
            call writeLog '[NOTICE]   A network adapter was found in your system'
            call writeLog '[NOTICE]     Manufacturer ID: "' || mfg_id || '"  Device ID: "' || dev_id || '"'
            call writeLog
            call writeLog '[NOTICE]     Vendor: ' || vendor_name
            call writeLog '[NOTICE]     Device: ' || device_name
            call writeLog
            call writeLog '[FATAL ERROR] No eCS driver is listed for this device in'
            call writeLog '              "' || nic_lst || '"'
         END
         ELSE DO
            call writeLog '[NOTICE]   A network adapter was found in your system'
            call writeLog '[NOTICE]     Manufacturer ID: "' || mfg_id || '"  Device ID: "' || dev_id || '"'
            call writeLog
            call writeLog '[NOTICE]     Vendor: ' || vendor_name
            call writeLog
            call writeLog '[FATAL ERROR] No eCS driver is listed for this device in'
            call writeLog '              "' || nic_lst || '"'
         END
      END
      ELSE DO
         call writeLog '[NOTICE]   A network adapter was found in your system'
         call writeLog '[NOTICE]     Manufacturer ID: "' || mfg_id || '"  Device ID: "' || dev_id || '"'
         call writeLog
         call writeLog '[FATAL ERROR] No eCS driver is listed for this device in'
         call writeLog '              "' || nic_lst || '"'
      END
      call writeLog 'Exiting..'
      exit
   END
END

IF in_ibm_macs = 1 THEN DO
   call writeLog '[NOTICE]   Drivers for this NIC adapter have been identified'
END

call writeLog '           nic_nif (1): ' || nic_nif.1
call writeLog '[NOTICE]   Please wait, checking if driver files exist..'

source_NIF_File =       ibm_macs || '\' || nic_nif.1
IF STREAM(source_NIF_File,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '           ' || source_NIF_File || ' (File exists)'
END
ELSE DO
   call writeLog '[FATAL ERROR] File "' || source_NIF_File || '" not found'
   call writeLog 'Exiting..'
   exit
END
call writeLog '[NOTICE]   Please wait, identifying driver file name from "' || nic_nif.1 || '"..'

tempValue = ReadKey(source_NIF_file,'FILE','name')
mac_drv  = strip(tempValue)
call writeLog '           driver:      ' || mac_drv
IF (mac_drv = '') THEN DO
   call writeLog '[WARNING]  Could not read 'NAME' key in "' || nic_nif.1 || '"'
END

source_NIC_Driver = ibm_macs || '\' || mac_drv
IF STREAM(source_NIC_Driver,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '           ' || source_NIC_Driver || ' (File exists)'
END
ELSE DO
   call writeLog '[FATAL ERROR] File "' || source_NIC_Driver || '" not found'
   call writeLog 'Exiting..'
   exit
END

call writeLog '[NOTICE]   Preparing to copy files to temporary location..'

/* Delete the three files on the ramdrive, if they already exist */
IF STREAM(dest_NIC_Driver_name,'C','QUERY EXISTS')<>'' THEN DO
   '@DEL ' || dest_NIC_Driver_name || '> NUL'
END
IF STREAM(dest_NIF_File_name,'C','QUERY EXISTS')<>'' THEN DO
   '@DEL ' || dest_NIF_File_name || '> NUL'
END
IF STREAM(dest_protocolINI_name,'C','QUERY EXISTS')<>'' THEN DO
   '@DEL ' || dest_protocolINI_name || '> NUL'
END

/* The three files ( procotol.ini, driver file, & NIF should now be deleted - double check they don't exist */
IF STREAM(dest_NIC_Driver_name,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '[FATAL ERROR] File "' || dest_NIC_Driver_name || '" could not be deleted'
   call writeLog 'Exiting..'
   exit
END
IF STREAM(dest_NIF_File_name,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '[FATAL ERROR] File "' || dest_NIF_File_name || '" could not be deleted'
   call writeLog 'Exiting..'
   exit
END
IF STREAM(dest_protocolINI_name,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '[FATAL ERROR] File "' || dest_protocolINI_name || '" could not be deleted'
   call writeLog 'Exiting..'
   exit
END

/* Copy the two NIC files */

command = 'COPY ' || source_NIC_Driver || ' ' || dest_NIC_Driver_name || ' /V > NUL'
'@' || command
IF STREAM(dest_NIC_Driver_name,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '           ' || source_NIC_Driver || ' --> ' || dest_NIC_driver_name || '  (SUCCESSFUL)'
END
ELSE DO
   call writeLog '[FATAL ERROR] File "' || dest_NIC_Driver_name || '" could not be written'
   call writeLog 'Exiting..'
   exit
END

command = 'COPY ' || source_NIF_File || ' ' || dest_NIF_File_name || ' /V > NUL'
'@' || command
IF STREAM(dest_NIF_File_name,'C','QUERY EXISTS')<>'' THEN DO
   call writeLog '           ' || source_NIF_File || ' --> ' || dest_NIF_File_name || '  (SUCCESSFUL)'
END
ELSE DO
   call writeLog '[FATAL ERROR] File "' || dest_NIF_File_name || '" could not be written'
   call writeLog 'Exiting..'
   exit
END

/* Build the protocol.ini file */
call writeLog '[NOTICE]   Please wait, creating protocol.ini file ( "' || dest_protocolINI_name || '")..'

driverSectionName = left(mac_drv, length(mac_drv) - 4)

call writeLog '[NOTICE]     Reading section "' || driverSectionName || '" from "' || source_NIF_File || '"'

tempValue = readKey(dest_NIF_File_name, driverSectionName, 'drivername')
drivername = strip(tempValue)
call writeLog '             drivername:  ' || drivername

IF (drivername = '') THEN DO
   call writeLog '[WARNING]    Could not read "' || driverSectionName || '" section in "' || nic_nif.1 || '"'
   call writeLog '[NOTICE]     Using first section in "' || nic_nif.1 || '"'
   /* Parse the first line, hopefully that'll be it */
   pos = 0
   size = MeasureINIFile(dest_NIF_File_name)
   FLAG_first_section_found = 0
   DO WHILE (pos < size)
      pos = pos + 1
      tempValue = readLine(dest_NIF_File_name, pos)
      tempValue = strip(tempValue)
      IF (left(tempvalue,1) = "[") THEN DO
         IF (right(tempvalue,1) = "]") THEN DO
            tempvalue = left(tempvalue, length(tempvalue) - 1)
            tempvalue = right(tempvalue, length(tempvalue) - 1)
         END
      END
      driverSectionName = strip(tempValue)
      call writeLog '      driverSectionName:  ' || driverSectionName
      tempValue = readKey(dest_NIF_File_name, driverSectionName, 'drivername')
      IF (tempValue <> '') THEN DO
         drivername = strip(tempValue)
         call writeLog '             drivername:  ' || drivername
         pos = size
      END
   END
      IF (drivername = '') THEN DO
         call writeLog '[FATAL ERROR] Could not read "drivername" key in "' || driverSectionName || '" section in "' || nic_nif.1 || '"'
         exit
      END
END

value = STREAM(dest_protocolINI_name,'C','OPEN')
value = lineout(dest_protocolINI_name, '; eCS PROTOCOL.INI file', 1)
value = STREAM(dest_protocolINI_name,'C','CLOSE')

IF STREAM(dest_protocolINI_name,'C','QUERY EXISTS')='' THEN DO
   call writeLog '[FATAL ERROR] File "' || dest_protocolINI_name || '" could not be created'
   call writeLog 'Exiting..'
   exit
END

FLAG_install_TCPIP = 0
FLAG_install_NetBeUI = 0
FLAG_install_TCPBeUI = 0
FLAG_install_SNA = 0

/* Determine what protocols to install, look for flag files in the ramdrive */

protocol_words = '[NOTICE]     Binding the following protocols: '

/* TCP/IP */
tcp_flag1 = ramdrive || '\' || 'install_tcpip.flag'
tcp_flag2 = ramdrive || '\' || 'inst_tcp.flg'
IF STREAM(tcp_flag1,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_TCPIP = 1
   protocol_words = protocol_words || 'TCP/IP  '
END
IF STREAM(tcp_flag2,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_TCPIP = 1
   protocol_words = protocol_words || 'TCP/IP  '
END

/* NetBeUI */
netb_flag1 = ramdrive || '\' || 'install_netbeui.flag'
netb_flag2 = ramdrive || '\' || 'instnetb.flg'
IF STREAM(netb_flag1,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_NetBeUI = 1
   protocol_words = protocol_words || 'NetBeUI  '
END
IF STREAM(netb_flag2,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_NetBeUI = 1
   protocol_words = protocol_words || 'NetBeUI  '
END

/* TCPBeUI */
tcpb_flag1 = ramdrive || '\' || 'install_tcpbeui.flag'
tcpb_flag2 = ramdrive || '\' || 'insttcpb.flg'
IF STREAM(tcpb_flag1,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_TCPBeUI = 1
   protocol_words = protocol_words || 'TCPBeUI  '
END
IF STREAM(tcpb_flag2,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_TCPBeUI = 1
   protocol_words = protocol_words || 'TCPBeUI  '
END

/* SNA */
sna_flag1 = ramdrive || '\' || 'install_sna.flag'
sna_flag2 = ramdrive || '\' || 'inst_sna.flg'
IF STREAM(netb_flag1,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_SNA = 1
   protocol_words = protocol_words || 'SNA  '
END
IF STREAM(netb_flag2,'C','QUERY EXISTS')<>'' THEN DO
   FLAG_install_SNA = 1
   protocol_words = protocol_words || 'SNA  '
END

call writeLog protocol_words
IF (FLAG_install_TCPIP = 0) THEN DO
   IF (FLAG_install_NetBeUI = 0) THEN DO
      IF (FLAG_install_TCPBeUI = 0) THEN DO
         IF (FLAG_install_SNA = 0) THEN DO
            call writeLog '[WARNING]    No FLAG files were detected for protocols to bind'
            call writeLog
            call writeLog '[INPUT]      Press the number key to the left of the protocol to toggle it'
            call writeLog '             ON or OFF.  Selected protocols will have [ ] brackets placed'
            call writeLog '             around them'
            call writeLog '             When you are finished selecting, press zero (0)'
            parse value SysCurPos() with x y
            call writeLog '             1  TCP/IP  2  NetBeUI  3  TCPBeUI  4  SNA'
            parse value SysCurPos() with newX newY
            IF (newX = x) THEN DO
               call SysCurPos newX - 1, y
            END
            FLAG_do_loop = 1
            choices = '1 2 3 4 0'
            DO WHILE (FLAG_do_loop = 1)
               IF (chars() <> 0) THEN DO
                  temp = sysgetkey('NOECHO')
                  temp = translate(temp)
                  FLAG_valid_key_pressed = 0
                  /* Check if they pressed a valid key */
                  pos = wordpos(temp, choices)
                  IF (pos <> 0) THEN DO
                     IF (pos = 5) THEN DO
                        FLAG_do_loop = 0
                        protocol_words = '[NOTICE]     Binding the following protocols: '
                        IF (FLAG_install_TCPIP = 1) THEN protocol_words = protocol_words || 'TCP/IP  '
                        IF (FLAG_install_NetBeUI = 1) THEN protocol_words = protocol_words || 'NetBeUI  '
                        IF (FLAG_install_TCPBeUI = 1) THEN protocol_words = protocol_words || 'TCPBeUI  '
                        IF (FLAG_install_SNA = 1) THEN protocol_words = protocol_words || 'SNA  '
                        call writeLog protocol_words
                     END
                     /* call Beep 400, 100 */
                     /****************************/
                     IF (pos = 1) THEN DO
                        IF (FLAG_install_TCPIP = 0) THEN DO
                           FLAG_install_TCPIP = 1
                           pos = 0
                        END
                     END
                     IF (pos = 1) THEN DO
                        IF (FLAG_install_TCPIP = 1) THEN DO
                           FLAG_install_TCPIP = 0
                           pos = 0
                        END
                     END
                     /****************************/
                     IF (pos = 2) THEN DO
                        IF (FLAG_install_NetBeUI = 0) THEN DO
                           FLAG_install_NetBeUI = 1
                           pos = 0
                        END
                     END
                     IF (pos = 2) THEN DO
                        IF (FLAG_install_NetBeUI = 1) THEN DO
                           FLAG_install_NetBeUI = 0
                           pos = 0
                        END
                     END
                     /****************************/
                     IF (pos = 3) THEN DO
                        IF (FLAG_install_TCPBeUI = 0) THEN DO
                           FLAG_install_TCPBeUI = 1
                           pos = 0
                        END
                     END
                     IF (pos = 3) THEN DO
                        IF (FLAG_install_TCPBeUI = 1) THEN DO
                           FLAG_install_TCPBeUI = 0
                           pos = 0
                        END
                     END
                     /****************************/
                     IF (pos = 4) THEN DO
                        IF (FLAG_install_SNA = 0) THEN DO
                           FLAG_install_SNA = 1
                           pos = 0
                        END
                     END
                     IF (pos = 4) THEN DO
                        IF (FLAG_install_SNA = 1) THEN DO
                           FLAG_install_SNA = 0
                           pos = 0
                        END
                     END
                     /****************************/
                     /* SAY FLAG_install_TCPIP || '|' || FLAG_install_NetBeUI  || '|' || FLAG_install_TCPBeUI  || '|' || FLAG_install_SNA */
                     string = '             1  TCP/IP  2  NetBEUI  3  TCPBeUI  4  SNA'
                     string = '             '
                     IF (FLAG_install_TCPIP = 0) THEN DO
                        string = string || '1  TCP/IP '
                     END
                     ELSE
                     DO
                        string = string || '1 [TCP/IP]'
                     END
                     IF (FLAG_install_NetBeUI = 0) THEN string = string || ' 2  NetBeUI '
                     ELSE string = string || ' 2 [NetBeUI]'
                     IF (FLAG_install_TCPBeUI = 0) THEN string = string || ' 3  TCPBeUI '
                     ELSE string = string || ' 3 [TCPBeUI]'
                     IF (FLAG_install_SNA = 0) THEN string = string || ' 4  SNA '
                     ELSE string = string || ' 4 [SNA]'
                     SAY string
                     parse value SysCurPos() with newX newY
                     IF (newX = x) THEN DO
                        call SysCurPos newX - 1, y
                     END
                  END
               END
            END
         END
      END
   END
END


/* Build the protocol.ini file */

   INIFile = dest_protocolINI_name
   sectionName='PROT_MAN'
   keyName='DRIVERNAME'
   keyValue='PROTMAN$'
   value = updateINI(INIFile, sectionName, keyName, keyValue)
/*********************************************************/
IF (FLAG_install_TCPIP <> 0) THEN DO
   protocolFileName = 'TCPIP.NIF'
   boundNIC = 'eCSdrv_nif'
   call addProtocol INIFile protocolFileName boundNIC
END
/*********************************************************/
IF (FLAG_install_NetBeUI <> 0) THEN DO
   protocolFileName = 'NetBEUI.NIF'
   boundNIC = 'eCSdrv_nif'
   call addProtocol INIFile protocolFileName boundNIC
END
/*********************************************************/
IF (FLAG_install_TCPBeUI <> 0) THEN DO
   protocolFileName = 'TCPBeUI.NIF'
   boundNIC = 'eCSdrv_nif'
   call addProtocol INIFile protocolFileName boundNIC
END
/*********************************************************/
IF (FLAG_install_SNA <> 0) THEN DO
   protocolFileName = 'LANDD.NIF'
   boundNIC = 'eCSdrv_nif'
   call addProtocol INIFile protocolFileName boundNIC
END
/*********************************************************/
   sectionName='IBMLXCFG'
   keyName='eCSdrv_nif'
   keyValue=NIF_File_name
   value = updateINI(INIFile, sectionName, keyName, keyValue)

   sectionName='eCSdrv_nif'
   keyName='DriverName'
   keyValue=drivername
   value = updateINI(INIFile, sectionName, keyName, keyValue)

/* Check for mandatory NIF driver sections */

   call writeLog '[NOTICE]     Checking for mandatory protocol.ini sections in "' || nic_nif.1 || '"'
   size = MeasureINIFile(dest_NIF_File_name)
   FLAG_mandatory_section_found = 0
   pos = 0
   DO WHILE (pos < size)
      pos = pos + 1
      tempValue = readLine(dest_NIF_File_name, pos)
      tempValue = strip(tempValue)
      IF (left(tempvalue,1) = "[") THEN DO
         IF (right(tempvalue,1) = "]") THEN DO
            tempvalue = left(tempvalue, length(tempvalue) - 1)
            tempvalue = right(tempvalue, length(tempvalue) - 1)
            driverSectionName = strip(tempValue)
            /* SAY '      driverSectionName:  ' || driverSectionName */
            tempValue = readKey(dest_NIF_File_name, driverSectionName, 'optional')
            tempValue = strip(tempValue)
            tempValue = translate(tempValue)
            IF (tempValue = 'NO') THEN DO
               call writeLog '               optional:  ' || driverSectionName || ' (NO)'
               tempValue = readKey(dest_NIF_File_name, driverSectionName, 'default')
               tempValue = strip(tempValue)
               call writeLog '          default value:  ' || tempValue
               IF (tempValue <> '') THEN DO
                  sectionName='eCSdrv_nif'
                  keyName=driverSectionName
                  keyValue=tempValue
                  call writeLog '[NOTICE]     Adding "' || keyName || '" key with value of "' || tempValue || '" to "' || dest_NIF_File_name || '"'
                  value = updateINI(INIFile, sectionName, keyName, keyValue)
                  FLAG_mandatory_section_found = 1
               END
            END
         END
      END
   END

IF (FLAG_mandatory_section_found = 0) THEN DO
         call writeLog '[NOTICE]     No mandatory protocol.ini sections found for "' || nic_nif.1 || '"'
END

call writeLog '[NOTICE]   Creation of protocol.ini file ( "' || dest_protocolINI_name || '") is complete'
call writeLog '[FINISHED] DONE!'

exit

/* ISA NIC Detection via RM View ID

 16.  PnP SMC8416: SMC EtherEZ (8416)
               IRQ: 7
*/

/* ISA NIC Detection via Daniel's ISA NIC detector 
[C:\vp21\out.os2]isanicprobe
eComStation ISA NIC Detection Tool v0.003

NIC: SMC EtherEZ
I/O: 340
NIF: ethos2at.nif

[C:\vp21\out.os2]

*/


/***************************************************/

/* Ugly hack because some versions of REXX don't support this function */
SysGetFileDateTime:
return 0

Build_device_list:
bus_device. = ''
bus_device.0 = 0
oem_device. = ''
oem_device.0 = 0

/* Query first with PCI.EXE */
pci_view = Stream(current_dir || '\pci.exe', 'c', 'query exists')
pci_arg = ''
If pci_view <> '' Then
   Call Query_devices pci_view, pci_arg       /* build device id list */

/* Query second with PCIPAS.EXE */
pci_view = Stream(current_dir || '\pcipas.exe', 'c', 'query exists')
pci_arg = ''
If pci_view <> '' Then
   Call Query_devices pci_view, pci_arg

/* Query last with RMVIEW.EXE, as it can identify hardware that is installed, but deactivated - (like Realtek NIC chips) on the motherboard in a sound card/NIC/modem adapter */
pci_view = SysSearchPath('path', 'rmview.exe')
pci_arg = '/dc'
If pci_view <> '' Then
   Call Query_devices pci_view, pci_arg

Call Check_devices                           /* check for NIC devices */
Return

/* query devices */
Query_devices:
Parse Arg pci_view, pci_arg, .
pci_prog = Translate(Filespec('name', pci_view))
dev_queue = Rxqueue('create')
old_queue = Rxqueue('set', dev_queue)
Address cmd '@' || pci_view pci_arg '| rxqueue' dev_queue

/* get device ids */
found_device = 0
Do i = 1 Until Queued() = 0
   mfg_no = ''
   Parse Pull line
   line = Space(line)
   Select
      When Abbrev(pci_prog, 'PCI.') Then    /* handle output from PCI */
         Do
            Parse Var line num ' | ' bus_vendor_id ' | ' bus_device_id ' | ' dev_class ' | ' .
            If Datatype(num, 'W') & Abbrev(dev_class, '200') Then do
               found_device = 1
               /* say '[PCI.EXE] bus_vendor_id: ' || bus_vendor_id || '  bus_device_id: ' || bus_device_id || '  dev_class: ' || dev_class */
            End
         End
      When Abbrev(pci_prog, 'PCIP') Then /* handle output from PCIPAS */
         Do
            If Abbrev(line, 'Vendor') Then
               Do
                  Parse Var line . bus_vendor_id .
                  bus_vendor_id = Left(bus_vendor_id, Length(bus_vendor_id) - 1)
                  Do Until Abbrev(line, 'PCI Class')
                     Parse Pull line
                     line = Space(line)
                     If Abbrev(line, 'Device') Then
                        Do
                           Parse Var line . bus_device_id .
                           bus_device_id = Left(bus_device_id, Length(bus_device_id) - 1)
                        End
                     If Abbrev(line, 'PCI Class') Then
                        Do
                           Parse Var line . 'Class' dev_class .
                           If Abbrev(dev_class, 'Network') Then
                              found_device = 1
                              /* say '[PCIPAS] bus_vendor_id: ' || bus_vendor_id || '  bus_device_id: ' || bus_device_id || '  dev_class: ' || dev_class */
                        End
                  End
               End
         End
      When Abbrev(pci_prog, 'RMV') Then  /* handle output from RMVIEW */
         Do
            If Abbrev(line, 'Detected Hardware Function') Then
               Do
                  Parse Var line . 'PCI Device' dev_class .
                  If Abbrev(dev_class, '02') Then
                     Do Until Abbrev(line, 'IRQ Level')
                        Parse Pull line
                        line = Space(line)
                        If Abbrev(line, 'PCI Device ID:') Then
                           Do
                              Parse Var line . ': ' bus_device_id
                              bus_vendor_id = Substr(bus_device_id, 1, 4)
                              bus_device_id = Substr(bus_device_id, 5)
                              found_device = 1
                              /* say '[RMVIEW] bus_vendor_id: ' || bus_vendor_id || '  bus_device_id: ' || bus_device_id || '  dev_class: ' || dev_class */
                           End
                        If Abbrev(line, 'PnP Device ID:') Then
                           Do
                              Parse Var line . ':' bus_device_id
                              Call Add_bus_device bus_device_id
                           End
                     End
               End
         End
      Otherwise Nop
   End
   If found_device Then
      Do
         bus_vendor_id = Right(Translate(bus_vendor_id), 4, '0')
         bus_device_id = Right(Translate(bus_device_id), 4, '0')
         /* SAY '[Adding device] bus_vendor_id: "' || bus_vendor_id || '"  bus_device_id: "' || bus_device_id || '"' */
         Call Add_bus_device bus_vendor_id || bus_device_id
         found_device = 0
      End
End
Call Rxqueue 'set', old_queue
Call Rxqueue 'delete', new_queue
Return

/* add non-duplicate bus device ids */
Add_bus_device: Procedure Expose bus_device. oem_device.
save_trace = Trace('N')
Parse Arg bus_device_id .
Do i = 1 To bus_device.0
   If bus_device_id = bus_device.i Then
      Return
End
i = bus_device.0 + 1
bus_device.i = bus_device_id
bus_device.0 = i
Trace(save_trace)
Return

/* check the ids found */
Check_devices:

Call Clear_nic_table                               /* clear NIC table */

/* scan for NIC cards */
Do i = 1 To bus_device.0
   test = bus_device.i
   rc = SysFileSearch(test, nic_lst, 'nic_found.')
   If rc = 0 & nic_found.0 <> 0 Then
      Do j = 1 To nic_found.0
         Call Nic_load nic_found.j              /* get NIC table info */
      End
End
Return

/* clear NIC table */
Clear_nic_table:
save_trace = Trace('N')
mfg_no. = ''
mfg_no.0 = 0
oem_no. = ''
oem_no.0 = 0
nic_title. = ''
nic_title.0 = 0
max_title = 0
nic_dir. = ''
nic_dir.0 = 0
max_dir = 0
nic_nif. = ''
nic_nif.0 = 0
max_nif = 0
Trace(save_trace)
Return

/* get NIC table info */
Nic_load:
save_trace = Trace('N')
Parse Arg nic_found
If \Abbrev(nic_found, ';') Then
   Do
      Parse Var nic_found mfg_no '|' oem_no '|' nic_nif '|' nic_title
      nic_dir = Left(nic_nif, Lastpos('\', nic_nif) - 1)
      nic_nif = Substr(nic_nif, Lastpos('\', nic_nif) + 1)
      k = mfg_no.0 + 1
      mfg_no.k = mfg_no
      mfg_no.0 = k
      k = oem_no.0 + 1
      oem_no.k = oem_no
      oem_no.0 = k
      k = nic_title.0 + 1
      nic_title.k = Space(nic_title)
      nic_title.0 = k
      If Length(nic_title.k) > max_title Then
         max_title = Length(nic_title.k)
      k = nic_dir.0 + 1
      nic_dir.k = Space(nic_dir)
      nic_dir.0 = k
      If Length(nic_dir.k) > max_dir Then
         max_dir = Length(nic_nif.k)
      k = nic_nif.0 + 1
      nic_nif.k = Space(nic_nif)
      nic_nif.0 = k
      If Length(nic_nif.k) > max_nif Then
         max_nif = Length(nic_nif.k)
   End
Trace(save_trace)
Return

/* select a NIC driver */
Select_nic:
save_trace = Trace('N')
Parse Arg detail .
detail = Abbrev(detail, 'D')
max_no = Length(mfg_no.0)
screen_list. = ''
screen_list.0 = 0
x = 0
Do i = 1 To mfg_no.0
   If detail | i = 1 Then
      Do
         x = x + 1; screen_list.x = Copies(' ', max_no) '  Chipset ID   ' 'Chip Mfg - Card Title'
      End
   x = x + 1; screen_list.x = Right(i, max_no) '-' Left(mfg_no.i, Length('Card Number') + 2) Left(nic_title.i, max_title)
   If Length('NIF File') > max_nif Then
      max_nif = Length('NIF File')
   Parse Value(Find_nif(ibm_macs, nic_nif.i)) With in_ibm_macs ibm_macs_nif '|' ibm_macs_driver '|' ibm_copyfile
   Parse Value(Find_nif(current_dir || '\' || nic_dir.i, nic_nif.i)) With in_nic_macs nic_macs_nif '|' nic_macs_driver '|' nic_copyfile
   x = x + 1; screen_list.x = Copies(' ', max_no + 2) Left('NIF File', max_nif + 1) 'NIF & Driver Information'
   If in_ibm_macs Then
      Do
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Left(nic_nif.i, max_nif + 1) ibm_macs_nif
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) ibm_macs_driver
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) ibm_copyfile
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_macs_nif
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_macs_driver
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_copyfile
         /* Next line commented out for rev. 18 */
         /* x = x + 1; screen_list.x = '' */
      End
   Else
      Do
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Left(nic_nif.i, max_nif + 1) nic_macs_nif
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_macs_driver
         x = x + 1; screen_list.x = Copies(' ', max_no + 2) Copies(' ', max_nif + 1) nic_copyfile
         /* Next line commented out for rev. 18 */
         /* x = x + 1; screen_list.x = '' */
      End
End
screen_list.0 = x
Do i = 1 To screen_list.0
   call writeLog screen_list.i
End
If in_nic_macs Then
   Do
      call writeLog 'Enter the number of the NIC you want to process, or press enter to bypass'
      Parse Upper Pull ans .
   End
Else
   ans = ''
Trace(save_trace)
Return ans

Find_nif: Procedure
nicpak_web = 'http://pws.prserv.net/mckinnis/nicpak/'
save_trace = Trace('N')
Parse Arg dir, nif
version = ''
nic_stamp = ''
driver = ''
copyfile = ''
rc = SysFileTree(dir || '\' || nif, 'nic_loc.', 'fo')
If rc = 0 & nic_loc.0 > 0 Then
   Do
      located = 1
      nic_loc = Space(nic_loc.1)
   End
Else
   Do
      located = 0
      Select
         When Substr(dir, Lastpos('\', dir) + 1) = 'current' Then
            nic_loc = 'An eCS driver for this chipset is known'
         When Substr(dir, Lastpos('\', dir) + 1) = 'nothere' Then
            nic_loc = 'No eCS driver for this chipset has been found'
         Otherwise
            Do
               nic_loc = 'This NIF and driver can be downloaded from'
               nic_zip = Substr(dir, Lastpos('\', dir) + 1) || '.zip'
               driver = nicpak_web || nic_zip
            End
      End
   End
If located Then
   Do
      /* locate version */
      rc = SysFileSearch('version', nic_loc, 'version.')
      If rc = 0 & version.0 <> 0 Then
         Do i = 1 To version.0
            If \Abbrev(Translate(Space(version.i)), 'VERSION') Then
               Iterate i
            Parse Value(Space(version.i)) With . '=' Version
            version = 'Version=' || Space(Version)
         End
      Else
         version = 'Version=unknown'
      /* locate driver name */
      rc = SysFileSearch('name', nic_loc, 'driver.')
      If rc = 0 & driver.0 <> 0 Then
         Do i = 1 To driver.0
            If \Abbrev(Translate(Space(driver.i)), 'NAME') Then
               Iterate i
            Parse Value(Space(driver.i)) With . '=' driver
            driver = Space(driver)
            rc = SysFileTree(dir || '\' || driver, 'driver_loc.', 'fso')
            If rc = 0 & driver_loc.0 > 0 Then
               Do
                  driver = driver_loc.1
                  driver_stamp = SysGetFileDateTime(driver, 'w')
                  driver = 'Driver=' || driver driver_stamp
               End
            Else
               driver = 'Driver=' || driver '(not found in' dir || ')'
         End
      Else
         driver = 'Driver=unknown'
      /* locate copyfile */
      rc = SysFileSearch('copyfile', nic_loc, 'copyfile.')
      If rc = 0 & copyfile.0 <> 0 Then
         Do i = 1 To copyfile.0
            If \Abbrev(Translate(Space(copyfile.i)), 'COPYFILE') Then
               Iterate i
            Parse Value(Space(copyfile.i)) With . '=' copyfile
            copyfile = Space(copyfile)
            rc = SysFileTree(dir || '\' || copyfile, 'copyfile_loc.', 'fso')
            If rc = 0 & copyfile_loc.0 > 0 Then
               Do
                  copyfile = copyfile_loc.1
                  copyfile_stamp = SysGetFileDateTime(copyfile, 'w')
                  copyfile = 'Copyfile=' || copyfile copyfile_stamp
               End
            Else
               copyfile = 'Copyfile=' || copyfile '(not found in' dir || ')'
         End
      Else
         copyfile = 'Copyfile=none'
      nic_stamp = SysGetFileDateTime(nic_loc, 'w')
   End
Trace(save_trace)
Return located nic_loc version nic_stamp '|' driver '|' copyfile

/*****************************************************************************
 * UpdateINI           - procedure to search through an INI file and add
 *                       or update sections and keys as necessary
 *****************************************************************************/

UpdateINI: PROCEDURE
    PARSE ARG INIFile, sectionName, keyName, keyValue

    IF (COMPARE(ramdrive, 'RAMDRIVE') = 0) THEN DO
        /* Workaround to a bug in my version of REXX */
        ramdrive = left(INIFile, 2)
    END

    INIFileNew = ramdrive || '\INITemp.INI'

    IF STREAM(INIFileNew,"C","QUERY EXISTS")<>"" THEN DO
        '@del /n ' || INIFileNew || '>nul'
    END
    sectionLineStart = 0
    sectionLineEnds = 0
    keyLine = 0

    sectionLineStart = LocateSection(INIFile, sectionName)
    sectionLineEnds = LocateSectionEnd(INIFile, sectionName, sectionLineStart)
    keyLine = LocateKey(INIFile, keyName, sectionLineStart, sectionLineEnds)
    INIFileLength = measureINIFile(INIFile)

    IF (sectionLineStart > 0) THEN DO
        /* say 'Section ''' || sectionName || ''' found, looking for key ''' || keyName || '''' */
    END
    ELSE DO
        /* say 'Section ' || sectionName || ' not found; creating' */
        value = AddINISection(INIFile, sectionName)
        '@copy ' || INIFileNew || ' ' || INIFile '>nul'
        '@del ' || INIFileNew
        sectionLineStart = LocateSection(INIFile, sectionName)
        sectionLineEnds = LocateSectionEnd(INIFile, sectionName, sectionLineStart)
        keyLine = LocateKey(INIFile, keyName, sectionLineStart, sectionLineEnds)
        INIFileLength = measureINIFile(INIFile)
        /* say 'Section ' || sectionName || ' starts on line ' || sectionLineStart || ' and ends on line ' || sectionLineEnds */
    END

    IF (keyLine > 0) THEN DO
        /* say 'Key ' || keyName || ' found on line ' || keyLine */
        value = UpdateINIKey(INIFile, keyName, keyLine, keyValue)
        '@copy ' || INIFileNew || ' ' || INIFile '>nul'
        '@del ' || INIFileNew
        /* say '    Updated with Value=' || keyValue */
        return 1
    END
    ELSE DO
        /* say 'Key ' || keyName || ' not found; creating' */
        value = AddINIKey(INIFile, keyName, sectionLineStart, KeyValue)
        '@copy ' || INIFileNew || ' ' || INIFile '>nul'
        '@del ' || INIFileNew
        return 1
    END
    return 0

/*****************************************************************************
 * LocateSection       - procedure to search through an INI file and return
 *                       the line number that a section starts on
 *****************************************************************************/

LocateSection: PROCEDURE
    PARSE ARG INIFile, sectionName

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END
    lineNumber = 1

    sectionName = translate(sectionName,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        line = translate(line,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
        IF (compare(line, '['||sectionName||']') = 0) THEN DO
            /* say 'LINE ' || linenumber ||':  ' || line || '  SectionName: [' || sectionName || ']' */
            CALL STREAM INIFile,'C','CLOSE'
            return lineNumber
        END
        lineNumber = lineNumber + 1
    END
    /* say 'LocateSection: Section ' || sectionName || ' not found' */
    firstCharacter = substr(line,1,1)
    /* CALL STREAM INIFILE,'C','SEEK='LineNumber */

    CALL STREAM INIFile,'C','CLOSE'

    return 0

/*****************************************************************************
 * LocateSectionEnd    - procedure to search through an INI file and return
 *                       the line number that a section ends on
 *****************************************************************************/

LocateSectionEnd: PROCEDURE
    PARSE ARG INIFile, sectionName, firstline

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END

    sectionStart = firstline
    count = 0
    lineNumber = 0

    DO WHILE count < sectionStart
        line = LINEIN(INIFile)
        lineNumber = lineNumber + 1
        IF (LINES(INIFile) = 0) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            return lineNumber
        END
        count = count + 1
    END

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        lineNumber = lineNumber + 1
        firstCharacter = substr(line,1,1)
        IF (compare(firstCharacter, '[') = 0) THEN DO
            /*****************************************
            say 'LINE ' || linenumber ||':  ' || line
            *****************************************/
            CALL STREAM INIFile,'C','CLOSE'
            return lineNumber - 1
        END
    END


    CALL STREAM INIFile,'C','CLOSE'

    return linenumber

/*****************************************************************************
 * LocateKey        - procedure to the line that a key starts on in an INI
 *                       file
 *****************************************************************************/

LocateKey: PROCEDURE
    PARSE ARG INIFile, keyName, firstLine, sectionEnds

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END

    IF (firstLine = 0) THEN DO
        CALL STREAM INIFile,'C','CLOSE'
        return 0
    END

    sectionStart = firstLine
    count = 0
    lineNumber = 0

    DO WHILE count < sectionStart
        line = LINEIN(INIFile)
        lineNumber = lineNumber + 1
        IF (LINES(INIFile) = 0) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            say 'Key search, initial line count exceeded'
            return 0
        END
        count = count + 1
    END

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        /* Added to ignore initial whitespace on keys */
        line = strip(line)
        lineNumber = lineNumber + 1
        PARSE VAR line keytest'='entry
        keyName = translate(keyName,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
        keytest = translate(keyTest,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
        IF (lineNumber > sectionEnds ) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            /* It didn't find the key inside of the section */
            return 0
        END
        IF (compare(keyTest, keyName) = 0) THEN DO
            /* say 'LINE ' || lineNumber ||':  ' || line */
            CALL STREAM INIFile,'C','CLOSE'
            return lineNumber
        END
    END

    CALL STREAM INIFile,'C','CLOSE'
    /* say 'LocateKey: KeyName ' || keyName || ' not found!' */
    return 0

/*****************************************************************************
 * UpdateINIKey        - procedure to update a key, that already exists,
 *                       to the specified INI file
 *****************************************************************************/

UpdateINIKey: PROCEDURE
    PARSE ARG INIFile, keyName, keyLine, keyValue

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END

    IF (keyLine = 0) THEN DO
        say 'UpdateINIKey: Zero value for key line; terminating'
        CALL STREAM INIFile,'C','CLOSE'
        return 0
    END

    IF (COMPARE(ramdrive, 'RAMDRIVE') = 0) THEN DO
        /* Workaround to a bug in my version of REXX */
        ramdrive = left(INIFile, 2)
    END

    INIFileTemp = ramdrive || '\INITemp.INI'
    LineNumber = 0
    FLAG_updated = 0

    DO WHILE LINES(INIFile) > 0
        lineNumber = lineNumber + 1
        line = LINEIN(INIFile)
        IF (lineNumber = keyLine) THEN DO
            value = LINEOUT(INIFileTemp, keyname || '=' || keyValue)
            FLAG_updated = 1
        END
        ELSE DO
            value = LINEOUT(INIFileTemp, line)
        END
    END
    CALL STREAM INIFile,'C','CLOSE'
    CALL STREAM INIFileTemp,'C','CLOSE'

    return FLAG_updated

/*****************************************************************************
 * AddINIKey           - procedure to add a key, that does not exist,
 *                       to the specified INI file
 *****************************************************************************/

AddINIKey: PROCEDURE
    PARSE ARG INIFile, keyName, sectionLineStart, keyValue

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN
        DO
            SAY "Error: Unable to find '"||INIFile||"'."
            EXIT
        END

    IF (sectionLineStart = 0) THEN DO
        say 'AddINIKey: Zero value for section line start; terminating'
        CALL STREAM INIFile,'C','CLOSE'
        return 0
    END

    IF (COMPARE(ramdrive, 'RAMDRIVE') = 0) THEN DO
        /* Workaround to a bug in my version of REXX */
        ramdrive = left(INIFile, 2)
    END

    INIFileNew = ramdrive || '\INITemp.INI'
    LineNumber = 0

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        value = LINEOUT(INIFileNew, line)
        IF (lineNumber = sectionLineStart - 1) THEN DO
            value = LINEOUT(INIFileNew, keyname || '=' || keyValue)
            /* say 'Key ' || keyName || ' sucessfully added' */
        END
        lineNumber = lineNumber + 1
    END
    CALL STREAM INIFile,'C','CLOSE'
    CALL STREAM INIFileNew,'C','CLOSE'

    return 1

/*****************************************************************************
 * AddINISection       - procedure to add a section, that does not exist,
 *                       to the specified INI file
 *****************************************************************************/

AddINISection: PROCEDURE
    PARSE ARG INIFile, sectionName, INIFileLineEnds

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN
        DO
            SAY "Error: Unable to find '"||INIFile||"'."
            EXIT
        END

    IF (COMPARE(ramdrive, 'RAMDRIVE') = 0) THEN DO
        /* Workaround to a bug in my version of REXX */
        ramdrive = left(INIFile, 2)
    END

    INIFileTemp= ramdrive || '\INITemp.INI'
    LineNumber = 0

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        value = LINEOUT(INIFileTemp, line)
        lineNumber = lineNumber + 1
    END
    crlf='0d0a'x
    value = LINEOUT(INIFileTemp, '[' || sectionName || ']' || crlf)

    CALL STREAM INIFile,'C','CLOSE'
    CALL STREAM INIFileTemp,'C','CLOSE'
    return 1

/*****************************************************************************
 * MeasureINIFile      - procedure to find how many lines there are
 *                       in the specified INI file
 *****************************************************************************/

MeasureINIFile: PROCEDURE
    PARSE ARG INIFile

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN
        DO
            SAY "Error: Unable to find '"||INIFile||"'."
            EXIT
        END

    lineCount = 0

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        lineCount = lineCount + 1
    END
    CALL STREAM INIFile,'C','CLOSE'

    return lineCount

/*****************************************************************************
 * Standard REXX program error handler
 *****************************************************************************/
ERROR:
    SAY "Error" rc "in system call."
    SAY "Line number =" sigl
    SAY "Instruction = "||SOURCELINE(sigl)
    EXIT

FAILURE:
    SAY "System failure detected (error code ="rc")"
    SAY "Line number =" sigl
    SAY "Instruction = "||SOURCELINE(sigl)
    EXIT
/*****************************************************************************/

/*****************************************************************************
 * LocateKey        - procedure to find the line number that a key starts on
 *                       in an INI file
 *****************************************************************************/

LocateKey: PROCEDURE
    PARSE ARG INIFile, keyName, firstLine, sectionEnds

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END

    IF (firstLine = 0) THEN DO
        CALL STREAM INIFile,'C','CLOSE'
        return 0
    END

    sectionStart = firstLine
    count = 0
    lineNumber = 0

    DO WHILE count < sectionStart
        line = LINEIN(INIFile)
        lineNumber = lineNumber + 1
        IF (LINES(INIFile) = 0) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            say 'Key search, initial line count exceeded'
            return 0
        END
        count = count + 1
    END

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        lineNumber = lineNumber + 1
        PARSE VAR line keytest'='entry
        keyName = translate(keyName,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
        keytest = translate(keyTest,'ABCDEFGHIJKLMNOPQRSTUVWXYZ','abcdefghijklmnopqrstuvwxyz')
        IF (lineNumber > sectionEnds ) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            /* It didn't find the key inside of the section */
            return 0
        END
        IF (compare(keyTest, keyName) = 0) THEN DO
            /* say 'LINE ' || lineNumber ||':  ' || line */
            CALL STREAM INIFile,'C','CLOSE'
            return lineNumber
        END
    END

    CALL STREAM INIFile,'C','CLOSE'
    /* say 'LocateKey: KeyName ' || keyName || ' not found!' */
    return 0

/*****************************************************************************
 * ReadKey             - procedure to search through an INI file and return
 *                       the value of the key requested
 *****************************************************************************/

ReadKey: PROCEDURE
    PARSE ARG fileName, sectionName, keyName

    INIFile = fileName

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END

    startLine = LocateSection(INIFile, sectionName)
    endline = LocateSectionEnd(INIFile, sectionName, startline)
    lineNumber = LocateKey(INIFile, keyName, startLine, endline)
    line = ReadLine(INIFile, lineNumber)

    PARSE VAR line temp'='value
    return value

/*****************************************************************************
 * ReadLine            - procedure to search through an INI file and return
 *                       a line number
 *****************************************************************************/

ReadLine: PROCEDURE
    PARSE ARG INIFile, LinePos

    IF STREAM(INIFile,"C","QUERY EXISTS")="" THEN DO
        SAY "Error: Unable to find '"||INIFile||"'."
        EXIT
    END
    lineNumber = 1

    DO WHILE LINES(INIFile) > 0
        line = LINEIN(INIFile)
        IF (lineNumber = LinePos) THEN DO
            CALL STREAM INIFile,'C','CLOSE'
            return line
        END
        lineNumber = lineNumber + 1
    END
    /* say 'LocateSection: Section ' || sectionName || ' not found' */
    firstCharacter = substr(line,1,1)
    /* CALL STREAM INIFILE,'C','SEEK='LineNumber */

    CALL STREAM INIFile,'C','CLOSE'

    return 0

/*****************************************************************************
 * WaitTime            - procedure to pause for a specified number of seconds
 *                       it will terminate early if any key is pressed
 *****************************************************************************/

WaitTime: PROCEDURE
    PARSE ARG time

IF (time < 1) THEN DO
   SAY 'WaitTime: 'time' less than 1'
   return
END
IF (time > 59) THEN DO
   time = 60
END

parse value SysCurPos() with x y

string = substr('',1,time,'.') || substr('',1,60 - time)
SAY string

DO WHILE (TIME > 0)
   FLAG_chars_in_buffer = chars()
   IF (FLAG_chars_in_buffer <> 0) THEN DO
      DO WHILE (chars() <> 0)
         temp = sysgetkey('NOECHO')
      END
      call SysCurPos x, y
      return 1
   END
   call sysSleep(1)
   time = time - 1
   call SysCurPos x, y
   string = substr('',1,time,'.') || substr('',1,60 - time)
   SAY string
END

call SysCurPos x, y
return 0

/*****************************************************************************
 * setupLogs           - procedure to determine where log files go
 *
 *****************************************************************************/
setupLogs: PROCEDURE Expose logDir logFile ramdrive current_dir version

IF (ramdrive = '') THEN DO
   /* use the current directory to write log files */
   logDir = current_dir
   logFile = logDir || '\nicdetec.log'
   /* Delete a logfile if it exists */
   IF STREAM(logFile,'C','QUERY EXISTS')<>'' THEN DO
      '@DEL ' || logFile || '> NUL'
   END
   IF STREAM(logFile,'C','QUERY EXISTS')<>'' THEN DO
      SAY '[FATAL ERROR] Existing logfile "' || logfile || '" could not be deleted'
      SAY 'Exiting..'
      exit
   END
   /* Check to see that we can write a logfile there */
   value = STREAM(logFile,'C','OPEN')
   value = lineout(logFile, 'eCS nicdetec.cmd log file - Script Version ' || version, 1)
   value = lineout(logFile, 'logFile:     ' || logFile)
   value = lineout(logFile, 'logDir:      ' || logDir)
   value = lineout(logFile, 'ramdrive:    ' || ramdrive)
   value = lineout(logFile, 'current_dir: ' || current_dir)
   value = lineout(logFile, '')
   value = STREAM(logFile,'C','CLOSE')
   IF STREAM(logFile,'C','QUERY EXISTS')='' THEN DO
      SAY '[FATAL ERROR] logfile "' || logfile || '" could not be created'
      logFile = ''
      return
   END
   text = '[NOTICE]   Logfile "' || logFile || '" created'
   call writeLog(text)
END

IF (ramdrive <> '') THEN DO
   /* use the ramdrive to write log files */
   logDir = ramdrive
   logFile = logDir || '\nicdetec.log'
   /* Delete a logfile if it exists */
   IF STREAM(logFile,'C','QUERY EXISTS')<>'' THEN DO
      '@DEL ' || logFile || '> NUL'
   END
   IF STREAM(logFile,'C','QUERY EXISTS')<>'' THEN DO
      SAY '[FATAL ERROR] Existing logfile "' || logfile || '" could not be deleted'
      SAY 'Exiting..'
      exit
   END
   /* Check to see that we can write a logfile there */
   value = STREAM(logFile,'C','OPEN')
   value = lineout(logFile, 'eCS nicdetec.cmd log file - Script Version ' || version, 1)
   value = lineout(logFile, 'logFile:     ' || logFile)
   value = lineout(logFile, 'logDir:      ' || logDir)
   value = lineout(logFile, 'ramdrive:    ' || ramdrive)
   value = lineout(logFile, 'current_dir: ' || current_dir)
   value = lineout(logFile, '')
   value = STREAM(logFile,'C','CLOSE')
   IF STREAM(logFile,'C','QUERY EXISTS')='' THEN DO
      SAY '[FATAL ERROR] logfile "' || logfile || '" could not be created'
      logFile = ''
      return
   END
   text = '[NOTICE]   Logfile "' || logFile || '" created'
   call writeLog(text)
END

return

/*****************************************************************************
 * writeLog            - procedure to write a line to the logFile
 *
 *****************************************************************************/
writeLog: PROCEDURE Expose logDir logFile
    PARSE ARG text

SAY text
/* Don't log anything if we don't have a valid logfile */
IF (logFile = '') THEN DO
  return
END

currentLogSize = measureINIFile(logFile)
   value = lineout(logFile, text)
newLogSize = measureINIFile(logfile)

IF (currentLogSize = newlogSize) THEN DO
   SAY '[WARNING]  Unable to write line to logfile "' || logFile || '"!'
   return 0
END

return 0

/*****************************************************************************
 * addProtocol         - procedure to add lines for a networking protocol
 *                       to the protocol.ini file
 *****************************************************************************/
addProtocol: PROCEDURE expose ibm_protocol logDir logFile
    PARSE ARG protocolINIFile protocolFileName boundNIC

fullProtocolFileName = ibm_protocol || '\' || protocolFileName

IF STREAM(fullProtocolFileName,'C','QUERY EXISTS')='' THEN DO
   call writeLog '[WARNING]  Unable to fine protocol file ' || fullProtocolFileName
   return
END

IF STREAM(protocolINIFile,'C','QUERY EXISTS')='' THEN DO
   call writeLog '[WARNING]  Unable to fine protocol file ' || protocolINIFile
   return
END

call writeLog '[NOTICE]     Attempting to add network protocol from file'
call writeLog '             "' || fullProtocolFileName || '"'
call writeLog '[NOTICE]     Please wait, looking for master section of "' || ProtocolFileName || '"...'
/* Parse the file line by line to find the master section */
pos = 0
size = MeasureINIFile(fullProtocolFileName)

masterSectionName = ''

DO WHILE (pos < size)
   pos = pos + 1
   tempValue = readLine(fullProtocolFileName, pos)
   tempValue = strip(tempValue)
   IF (left(tempvalue,1) = "[") THEN DO
      IF (right(tempvalue,1) = "]") THEN DO
         tempvalue = left(tempvalue, length(tempvalue) - 1)
         tempvalue = right(tempvalue, length(tempvalue) - 1)
         /* call writeLog '        found section:  ' || tempvalue */
      END
   END
   protocolSectionName = strip(tempValue)
   /* call writeLog '    protocolSectionName:  ' || protocolSectionName */
   tempValue = readKey(fullProtocolFileName, protocolSectionName, 'type')
   tempValue = translate(tempvalue)
   tempValue = strip(tempValue)
   IF (tempValue = 'PROTOCOL') THEN DO
      drivername = strip(tempValue)
      /* call writeLog '              keyname:  ' || 'type' */
      /* call writeLog '             keyvalue:  ' || drivername */
      masterSectionName = protocolSectionName
      call writeLog '    masterSectionName:  ' || masterSectionName
      pos = size
   END
END

IF (masterSectionName = '') THEN DO
   call writeLog '[FATAL ERROR] Could not find master section in "' || fullProtocolFileName || '"'
   call writeLog 'Exiting..'
   exit
END

driverName = ''
tempValue = readKey(fullProtocolFileName, masterSectionName, 'drivername')
IF (tempValue <> '') THEN DO
   tempValue = strip(tempValue)
   driverName = tempValue
   call writeLog '           driverName:  ' || driverName
END

INIFile=protocolINIFile
sectionName='IBMLXCFG'
keyName=masterSectionName || '_nif'
keyValue=protocolFileName
value = updateINI(INIFile, sectionName, keyName, keyValue)

sectionName=masterSectionName || '_nif'
keyName='DriverName'
keyValue=driverName
value = updateINI(INIFile, sectionName, keyName, keyValue)

sectionName=masterSectionName || '_nif'
keyName='Bindings'
keyValue=boundNIC
value = updateINI(INIFile, sectionName, keyName, keyValue)

/* Parse the file line by line to identify keys */

call writeLog '[NOTICE]     Please wait, adding parameters for protocol "' || ProtocolFileName || '"..'

size = MeasureINIFile(fullProtocolFileName)
pos = size

/* Read values from the bottom to the top of the protocol file */
DO WHILE (pos > 0)
   pos = pos - 1
   FLAG_parse_section = 1
   tempValue = readLine(fullProtocolFileName, pos)
   tempValue = strip(tempValue)
   IF (left(tempvalue,1) = "[") THEN DO
      IF (right(tempvalue,1) = "]") THEN DO
         tempvalue = left(tempvalue, length(tempvalue) - 1)
         tempvalue = right(tempvalue, length(tempvalue) - 1)
         IF (tempvalue = masterSectionName) THEN DO
            FLAG_parse_section = 0
         END
         IF (tempvalue = 'FILE' ) THEN DO
            FLAG_parse_section = 0
         END
         IF (FLAG_parse_section = 1) THEN DO
            /* call writeLog '        found section:  ' || tempValue || ', parsing..' */
            keyName = tempValue
            keyValue = readKey(fullProtocolFileName, keyName, 'DEFAULT')
            keyValue = strip(keyValue)
            FLAG_add_line = 0
            IF (keyValue <> '') THEN DO
               /* call writeLog '         Should I add a ' || keyName || ' line to protocol.ini?' */
               unparsedValue = strip(keyValue)
               FLAG_add_line = 1
               virtualValue = readKey(fullProtocolFileName, keyName, 'VIRTUAL')
               virtualValue = strip(virtualValue)
               virtualValue = translate(virtualValue)
               IF (virtualValue = 'YES') THEN DO
                  /* Don't add it */
                  FLAG_add_line = 0
               END
               IF (FLAG_add_line = 1) THEN DO
                  /* call writeLog '                        YES' */
/********/
                  typeValue = readKey(fullProtocolFileName, keyName, 'TYPE')
                  typeValue = strip(typeValue)
                  typeValue = translate(typeValue)
                  /* call writeLog '                 TYPE: ' typeValue */
                  FLAG_processed_line = 0
                  IF (typeValue = 'STRING') THEN DO
                     /* Add it as is, it should have double quotes around it */
                     FLAG_processed_line = 1
                     sectionName=masterSectionName || '_nif'
                     keyName=keyName
                     keyValue=keyValue
                     /* call writeLog '                        Writing string...' || keyValue */
                     value = updateINI(INIFile, sectionName, keyName, keyValue)
                  END
                  IF (typeValue = 'HEXADECIMAL') THEN DO
                     /* Add a 0x to the front, it shouldn't have double quotes in the first place */
                     FLAG_processed_line = 1
                     sectionName=masterSectionName || '_nif'
                     keyName=keyName
                     keyValue='0x' || keyValue
                     /* call writeLog '                        Writing hex number...' || keyValue */
                     value = updateINI(INIFile, sectionName, keyName, keyValue)
                  END
                  IF (typeValue = 'DECIMAL') THEN DO
                     /* Remove the double quotes from the front and back */
                     FLAG_processed_line = 1
                     IF (left(keyValue,1) = '"') THEN DO
                        keyValue = right(keyValue, length(keyValue) - 1)
                     END
                     IF (right(keyValue,1) = '"') THEN DO
                        keyValue = left(keyValue, length(keyValue) - 1)
                     END
                     sectionName=masterSectionName || '_nif'
                     keyName=keyName
                     keyValue=keyValue
                     /* call writeLog '                        Writing number...' || keyValue */
                     value = updateINI(INIFile, sectionName, keyName, keyValue)
                  END
               END
/********/
               IF (FLAG_processed_line = 0) THEN DO
                  call writeLog '[WARNING]            Unable to identify type of value for line ' || keyName
               END

               IF (FLAG_add_line = 0) THEN DO
                  /* call writeLog '                        NO' */
               END
            END
         END
      END
   END
END


/*
Type      = String
StrLength = 1
Default   = "I"
ex:   ETHERAND_TYPE = "I"

TYPE = Hexadecimal
DEFAULT = 0
ex:   OS2TRACEMASK = 0x0
Always starts with '0x'

Type      = decimal
Default   = "130"
ex:   SESSIONS = 130

Couldn't find boolean in anything..

*/


/************
Make sure the protocol file exists                              DONE
Identify the master section                                     DONE
Identify the Drivername in the master section                   DONE
Add the '_nif' to the IBMLXCFG section of the protocol.ini      DONE
Add the _nif section to the protocol.ini                        DONE
Add the drivername to the _nif section                          DONE
Parse through the file, ignore the master and 'FILE' section    ....
   Add keys & values for sections that have default =           ....
      IGNORE ones that are 'virtual'                            DONE
      for hex values
      for string values
      for number values
      for boolean (?) values
Bind protocol to designated NIC                                 DONE
************/

call writeLog '[NOTICE]     Completed adding parameters for protocol "' || ProtocolFileName || '"'

return
